@@ -1,12 +1,10 @@
-#include <polarssl/rsa.h>
-#include <polarssl/base64.h>
-#include <sha.hh>
-#include <polarssl/entropy.h>
-#include <polarssl/ctr_drbg.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <boost/assign/std/vector.hpp> // for 'operator+=()'
 #include <boost/assign/list_of.hpp>
 #include <boost/make_shared.hpp>
-#include <boost/foreach.hpp>
+
 #include <boost/format.hpp>
 #include <p11-kit/p11-kit.h>
 
@@ -230,10 +228,10 @@ class Pkcs11Slot {
       }
       // check if we need to login
       if ((d_err = d_functions->C_GetTokenInfo(d_slot, &tokenInfo)) == 0) {
-        d_logged_in = ((tokenInfo.flags && CKF_LOGIN_REQUIRED) == CKF_LOGIN_REQUIRED);
+        d_logged_in = !((tokenInfo.flags & CKF_LOGIN_REQUIRED) == CKF_LOGIN_REQUIRED);
       } else {
         logError("C_GetTokenInfo");
-        throw PDNSException("Cannot get token info for slot " + boost::lexical_cast<std::string>(slot));
+        throw PDNSException("Cannot get token info for slot " + std::to_string(slot));
       }
     }
 
@@ -262,13 +260,13 @@ class Pkcs11Slot {
 
     pthread_mutex_t *m() { return &d_m; }
 
-    static boost::shared_ptr<Pkcs11Slot> GetSlot(const std::string& module, const string& tokenId);
+    static std::shared_ptr<Pkcs11Slot> GetSlot(const std::string& module, const string& tokenId);
     static CK_RV HuntSlot(const string& tokenId, CK_SLOT_ID &slotId, _CK_SLOT_INFO* info, CK_FUNCTION_LIST* functions);
 };
 
 class Pkcs11Token {
   private:
-    boost::shared_ptr<Pkcs11Slot> d_slot;
+    std::shared_ptr<Pkcs11Slot> d_slot;
 
     CK_OBJECT_HANDLE d_public_key;
     CK_OBJECT_HANDLE d_private_key;
@@ -281,7 +279,7 @@ class Pkcs11Token {
     std::string d_ecdsa_params;
 
     std::string d_label;
-
+    bool d_loaded;
     CK_RV d_err;
 
     void logError(const std::string& operation) const {
@@ -292,13 +290,11 @@ class Pkcs11Token {
     }
 
   public:
-    Pkcs11Token(const boost::shared_ptr<Pkcs11Slot>& slot, const std::string& label); 
+    Pkcs11Token(const std::shared_ptr<Pkcs11Slot>& slot, const std::string& label); 
     ~Pkcs11Token();
 
     bool Login(const std::string& pin) {
-      if (pin.empty()) return CKR_PIN_INVALID; // no empty pin.
-      Lock l(d_slot->m());
-
+      if (pin.empty()) return false; // no empty pin.
       if (d_slot->Login(pin) == true) {
         LoadAttributes();
       }
@@ -306,9 +302,15 @@ class Pkcs11Token {
       return LoggedIn();
     }
 
-    bool LoggedIn() const { return d_slot->LoggedIn(); }
+    bool LoggedIn() {
+      if (d_loaded == false && d_slot->LoggedIn() == true) {
+        LoadAttributes();
+      }
+      return d_slot->LoggedIn();
+    }
 
     void LoadAttributes() {
+      Lock l(d_slot->m());
       std::vector<P11KitAttribute> attr;
       std::vector<CK_OBJECT_HANDLE> key;
       attr.push_back(P11KitAttribute(CKA_CLASS, (unsigned long)CKO_PRIVATE_KEY));
@@ -369,9 +371,12 @@ class Pkcs11Token {
       } else {
         throw PDNSException("Cannot load attributes for PCKS#11 public key " + d_label);
       }
+
+      d_loaded = true;
     }
 
     int GenerateKeyPair(CK_MECHANISM_PTR mechanism, std::vector<P11KitAttribute>& pubAttributes, std::vector<P11KitAttribute>& privAttributes, CK_OBJECT_HANDLE_PTR pubKey, CK_OBJECT_HANDLE_PTR privKey) {
+      {
       Lock l(d_slot->m());
 
       size_t k;
@@ -380,13 +385,13 @@ class Pkcs11Token {
       privAttr = new CK_ATTRIBUTE[privAttributes.size()];
 
       k = 0;
-      BOOST_FOREACH(P11KitAttribute& attribute, pubAttributes) {
+      for(P11KitAttribute& attribute :  pubAttributes) {
         attribute.rattr(pubAttr+k);
         k++;
       }
 
       k = 0;
-      BOOST_FOREACH(P11KitAttribute& attribute, privAttributes) {
+      for(P11KitAttribute& attribute :  privAttributes) {
         attribute.rattr(privAttr+k);
         k++;
       }
@@ -395,6 +400,7 @@ class Pkcs11Token {
       logError("C_GenerateKeyPair");
       delete [] pubAttr;
       delete [] privAttr;
+      }
 
       if (d_err == 0) LoadAttributes();
 
@@ -502,7 +508,7 @@ class Pkcs11Token {
       attr = new CK_ATTRIBUTE[attributes.size()];
 
       k = 0;
-      BOOST_FOREACH(const P11KitAttribute& attribute, attributes) {
+      for(const P11KitAttribute& attribute :  attributes) {
         attribute.rattr(attr+k);
         k++;
       }
@@ -551,7 +557,7 @@ class Pkcs11Token {
       attr = new CK_ATTRIBUTE[attributes.size()];
 
       k = 0;
-      BOOST_FOREACH(P11KitAttribute &attribute, attributes) {
+      for(P11KitAttribute &attribute :  attributes) {
         attribute.wattr(attr+k);
         k++;
       }
@@ -611,11 +617,11 @@ class Pkcs11Token {
       return d_bits;
     }
 
-    static boost::shared_ptr<Pkcs11Token> GetToken(const std::string& module, const string& tokenId, const std::string& label);
+    static std::shared_ptr<Pkcs11Token> GetToken(const std::string& module, const string& tokenId, const std::string& label);
 };
 
-static std::map<std::string, boost::shared_ptr<Pkcs11Slot> > pkcs11_slots;
-static std::map<std::string, boost::shared_ptr<Pkcs11Token> > pkcs11_tokens;
+static std::map<std::string, std::shared_ptr<Pkcs11Slot> > pkcs11_slots;
+static std::map<std::string, std::shared_ptr<Pkcs11Token> > pkcs11_tokens;
 
 CK_RV Pkcs11Slot::HuntSlot(const string& tokenId, CK_SLOT_ID &slotId, _CK_SLOT_INFO* info, CK_FUNCTION_LIST* functions)
 {
@@ -652,7 +658,7 @@ CK_RV Pkcs11Slot::HuntSlot(const string& tokenId, CK_SLOT_ID &slotId, _CK_SLOT_I
 
   // see if we can find it with slotId
   try {
-    slotId = boost::lexical_cast<int>(tokenId);
+    slotId = std::stoi(tokenId);
     if ((err = functions->C_GetSlotInfo(slotId, info))) {
       L<<Logger::Warning<<"C_GetSlotList("<<slotId<<", info) = " << err << std::endl;
       return err;
@@ -660,17 +666,17 @@ CK_RV Pkcs11Slot::HuntSlot(const string& tokenId, CK_SLOT_ID &slotId, _CK_SLOT_I
     L<<Logger::Warning<<"Specifying PKCS#11 token by SLOT ID is deprecated and should not be used"<<std::endl;
     return 0;
   } catch (...) {
-    return CK_UNAVAILABLE_INFORMATION;
+    return CKR_SLOT_ID_INVALID;
   }
-  return CK_UNAVAILABLE_INFORMATION;
+  return CKR_SLOT_ID_INVALID;
 }
 
-boost::shared_ptr<Pkcs11Slot> Pkcs11Slot::GetSlot(const std::string& module, const string& tokenId) {
+std::shared_ptr<Pkcs11Slot> Pkcs11Slot::GetSlot(const std::string& module, const string& tokenId) {
   // see if we can find module
   std::string sidx = module;
   sidx.append("|");
   sidx.append(tokenId);
-  std::map<std::string, boost::shared_ptr<Pkcs11Slot> >::iterator slotIter;
+  std::map<std::string, std::shared_ptr<Pkcs11Slot> >::iterator slotIter;
   CK_RV err;
   CK_FUNCTION_LIST* functions;
 
@@ -692,37 +698,37 @@ boost::shared_ptr<Pkcs11Slot> Pkcs11Slot::GetSlot(const std::string& module, con
   CK_SLOT_ID slotId;
 
   if ((err = Pkcs11Slot::HuntSlot(tokenId, slotId, &info, functions))) {
-    throw PDNSException(std::string("Cannot find PKCS#11 token ") + tokenId + std::string(" on module ") + module + std::string(": error code ") + boost::lexical_cast<std::string>(err));
+    throw PDNSException(std::string("Cannot find PKCS#11 token ") + tokenId + std::string(" on module ") + module + std::string(": ") + boost::str( boost::format("%s (0x%X)") % p11_kit_strerror(err) % err));
   }
 
   // store slot
-  pkcs11_slots[sidx] = boost::make_shared<Pkcs11Slot>(functions, slotId);
+  pkcs11_slots[sidx] = std::make_shared<Pkcs11Slot>(functions, slotId);
 
   return pkcs11_slots[sidx];
 }
 
-boost::shared_ptr<Pkcs11Token> Pkcs11Token::GetToken(const std::string& module, const string& tokenId, const std::string& label) {
+std::shared_ptr<Pkcs11Token> Pkcs11Token::GetToken(const std::string& module, const string& tokenId, const std::string& label) {
   // see if we can find module
   std::string tidx = module;
   tidx.append("|");
-  tidx.append(boost::lexical_cast<std::string>(tokenId));
+  tidx.append(tokenId);
   tidx.append("|");
   tidx.append(label);
-  std::map<std::string, boost::shared_ptr<Pkcs11Token> >::iterator tokenIter;
+  std::map<std::string, std::shared_ptr<Pkcs11Token> >::iterator tokenIter;
   if ((tokenIter = pkcs11_tokens.find(tidx)) != pkcs11_tokens.end()) return tokenIter->second;
 
-  boost::shared_ptr<Pkcs11Slot> slot = Pkcs11Slot::GetSlot(module, tokenId);
-  pkcs11_tokens[tidx] = boost::make_shared<Pkcs11Token>(slot, label);
+  std::shared_ptr<Pkcs11Slot> slot = Pkcs11Slot::GetSlot(module, tokenId);
+  pkcs11_tokens[tidx] = std::make_shared<Pkcs11Token>(slot, label);
   return pkcs11_tokens[tidx];
 }
 
-Pkcs11Token::Pkcs11Token(const boost::shared_ptr<Pkcs11Slot>& slot, const std::string& label) {
+Pkcs11Token::Pkcs11Token(const std::shared_ptr<Pkcs11Slot>& slot, const std::string& label) {
   // open a session
   this->d_bits = 0;
   this->d_slot = slot;
   this->d_label = label;
   this->d_err = 0;
-  Lock l(d_slot->m());
+  this->d_loaded = false;
   if (this->d_slot->LoggedIn()) LoadAttributes();
 }
 
@@ -731,7 +737,7 @@ Pkcs11Token::~Pkcs11Token() {
 
 bool PKCS11ModuleSlotLogin(const std::string& module, const string& tokenId, const std::string& pin)
 {
-  boost::shared_ptr<Pkcs11Slot> slot;
+  std::shared_ptr<Pkcs11Slot> slot;
   slot = Pkcs11Slot::GetSlot(module, tokenId);
   if (slot->LoggedIn()) return true; // no point failing
   return slot->Login(pin);
@@ -747,7 +753,7 @@ void PKCS11DNSCryptoKeyEngine::create(unsigned int bits) {
   CK_MECHANISM mech;
   CK_OBJECT_HANDLE pubKey, privKey;
   CK_RV rv;
-  boost::shared_ptr<Pkcs11Token> d_slot;
+  std::shared_ptr<Pkcs11Token> d_slot;
   d_slot = Pkcs11Token::GetToken(d_module, d_slot_id, d_label);
   if (d_slot->LoggedIn() == false)
     if (d_slot->Login(d_pin) == false)
@@ -788,7 +794,7 @@ void PKCS11DNSCryptoKeyEngine::create(unsigned int bits) {
 
 std::string PKCS11DNSCryptoKeyEngine::sign(const std::string& msg) const {
   std::string result;
-  boost::shared_ptr<Pkcs11Token> d_slot;
+  std::shared_ptr<Pkcs11Token> d_slot;
   d_slot = Pkcs11Token::GetToken(d_module, d_slot_id, d_label);
   if (d_slot->LoggedIn() == false)
     if (d_slot->Login(d_pin) == false)
@@ -813,7 +819,7 @@ std::string PKCS11DNSCryptoKeyEngine::hash(const std::string& msg) const {
   mech.mechanism = dnssec2hmech[d_algorithm];
   mech.pParameter = NULL;
   mech.ulParameterLen = 0;
-  boost::shared_ptr<Pkcs11Token> d_slot;
+  std::shared_ptr<Pkcs11Token> d_slot;
   d_slot = Pkcs11Token::GetToken(d_module, d_slot_id, d_label);
   if (d_slot->LoggedIn() == false)
     if (d_slot->Login(d_pin) == false)
@@ -824,29 +830,19 @@ std::string PKCS11DNSCryptoKeyEngine::hash(const std::string& msg) const {
     // FINE! I'll do this myself, then, shall I?
     switch(d_algorithm) {
     case 5: {
-      SHA1Summer sha;
-      sha.feed(msg);
-      return sha.get();
+      return pdns_sha1sum(msg);
     }
     case 8: {
-      SHA256Summer sha;
-      sha.feed(msg);
-      return sha.get();
+      return pdns_sha256sum(msg);
     }
     case 10: {
-      SHA512Summer sha;
-      sha.feed(msg);
-      return sha.get();
+      return pdns_sha512sum(msg);
     }
     case 13: {
-      SHA256Summer sha;
-      sha.feed(msg);
-      return sha.get();
+      return pdns_sha256sum(msg);
     }
     case 14: {
-      SHA384Summer sha;
-      sha.feed(msg);
-      return sha.get();
+      return pdns_sha384sum(msg);
     }
     };
   };
@@ -854,7 +850,7 @@ std::string PKCS11DNSCryptoKeyEngine::hash(const std::string& msg) const {
 };
 
 bool PKCS11DNSCryptoKeyEngine::verify(const std::string& msg, const std::string& signature) const {
-  boost::shared_ptr<Pkcs11Token> d_slot;
+  std::shared_ptr<Pkcs11Token> d_slot;
   d_slot = Pkcs11Token::GetToken(d_module, d_slot_id, d_label);
   if (d_slot->LoggedIn() == false)
     if (d_slot->Login(d_pin) == false)
@@ -873,7 +869,7 @@ bool PKCS11DNSCryptoKeyEngine::verify(const std::string& msg, const std::string&
 
 std::string PKCS11DNSCryptoKeyEngine::getPubKeyHash() const {
   // find us a public key
-  boost::shared_ptr<Pkcs11Token> d_slot;
+  std::shared_ptr<Pkcs11Token> d_slot;
   d_slot = Pkcs11Token::GetToken(d_module, d_slot_id, d_label);
   if (d_slot->LoggedIn() == false)
     if (d_slot->Login(d_pin) == false)
@@ -886,7 +882,7 @@ std::string PKCS11DNSCryptoKeyEngine::getPubKeyHash() const {
 
 std::string PKCS11DNSCryptoKeyEngine::getPublicKeyString() const {
   std::string result("");
-  boost::shared_ptr<Pkcs11Token> d_slot;
+  std::shared_ptr<Pkcs11Token> d_slot;
   d_slot = Pkcs11Token::GetToken(d_module, d_slot_id, d_label);
   if (d_slot->LoggedIn() == false)
     if (d_slot->Login(d_pin) == false)
@@ -909,7 +905,7 @@ std::string PKCS11DNSCryptoKeyEngine::getPublicKeyString() const {
 };
 
 int PKCS11DNSCryptoKeyEngine::getBits() const {
-  boost::shared_ptr<Pkcs11Token> d_slot;
+  std::shared_ptr<Pkcs11Token> d_slot;
   d_slot = Pkcs11Token::GetToken(d_module, d_slot_id, d_label);
   if (d_slot->LoggedIn() == false)
     if (d_slot->Login(d_pin) == false)
@@ -924,7 +920,7 @@ DNSCryptoKeyEngine::storvector_t PKCS11DNSCryptoKeyEngine::convertToISCVector()
   outputs_t outputs;
 
   boost::assign::push_back(storvect)
-   (make_pair("Algorithm", boost::lexical_cast<std::string>(d_algorithm)))
+   (make_pair("Algorithm", std::to_string(d_algorithm)))
    (make_pair("Engine", d_module))
    (make_pair("Slot", d_slot_id))
    (make_pair("PIN", d_pin))
@@ -933,7 +929,7 @@ DNSCryptoKeyEngine::storvector_t PKCS11DNSCryptoKeyEngine::convertToISCVector()
 };
 
 void PKCS11DNSCryptoKeyEngine::fromISCMap(DNSKEYRecordContent& drc, stormap_t& stormap) {
-  drc.d_algorithm = atoi(stormap["algorithm"].c_str());
+  drc.d_algorithm = pdns_stou(stormap["algorithm"]);
   d_module = stormap["engine"];
   d_slot_id = stormap["slot"];
   boost::trim(d_slot_id);
@@ -941,16 +937,16 @@ void PKCS11DNSCryptoKeyEngine::fromISCMap(DNSKEYRecordContent& drc, stormap_t& s
   d_label = stormap["label"];
   // validate parameters
 
-  boost::shared_ptr<Pkcs11Token> d_slot;
+  std::shared_ptr<Pkcs11Token> d_slot;
   d_slot = Pkcs11Token::GetToken(d_module, d_slot_id, d_label);
   if (d_pin != "" && d_slot->LoggedIn() == false)
     if (d_slot->Login(d_pin) == false)
       throw PDNSException("Could not log in to token (PIN wrong?)");
 };
 
-DNSCryptoKeyEngine* PKCS11DNSCryptoKeyEngine::maker(unsigned int algorithm)
+std::shared_ptr<DNSCryptoKeyEngine> PKCS11DNSCryptoKeyEngine::maker(unsigned int algorithm)
 {
-  return new PKCS11DNSCryptoKeyEngine(algorithm);
+  return std::make_shared<PKCS11DNSCryptoKeyEngine>(algorithm);
 }
 
 // this is called during program startup
